@@ -8,6 +8,7 @@ from typing import TYPE_CHECKING
 
 from pdm.cli.commands.base import BaseCommand
 from pdm.cli.options import verbose_option
+from pdm.environments import BareEnvironment
 from pdm.exceptions import InstallationError
 from pdm.models.python import PythonInfo
 
@@ -116,18 +117,20 @@ class InstallCommand(BaseCommand):
         ver, python_file = get_download_link(version, implementation=implementation, arch=arch)
         with ui.open_spinner(f"Downloading [success]{ver}[/]") as spinner:
             destination = root / str(ver)
+            interpreter = destination / "bin" / "python3" if sys.platform != "win32" else destination / "python.exe"
             logger.debug("Installing %s to %s", ver, destination)
-            if not destination.exists():
+            env = BareEnvironment(project)
+            if not destination.exists() or not interpreter.exists():
+                shutil.rmtree(destination, ignore_errors=True)
                 destination.mkdir(parents=True, exist_ok=True)
                 with tempfile.NamedTemporaryFile() as tf:
                     tf.close()
-                    original_filename = download(python_file, tf.name, project.environment.session)
+                    original_filename = download(python_file, tf.name, env.session)
                     spinner.update(f"Installing [success]{ver}[/]")
                     install_file(tf.name, destination, original_filename)
 
-        interpreter = destination / "bin" / "python3" if sys.platform != "win32" else destination / "python.exe"
         if not interpreter.exists():
-            raise InstallationError("Installation failed")
+            raise InstallationError("Installation failed, please try again.")
 
         python_info = PythonInfo.from_path(interpreter)
         ui.echo(f"[success]Successfully installed[/] {python_info.implementation}@{python_info.version}")
