@@ -2,11 +2,12 @@ from __future__ import annotations
 
 import contextlib
 import hashlib
+import operator
 import os
 import re
 import shutil
 import sys
-from functools import cached_property
+from functools import cached_property, reduce
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, cast
 
@@ -16,13 +17,13 @@ from tomlkit.items import Array
 from pdm import termui
 from pdm._types import RepositoryConfig
 from pdm.exceptions import NoPythonVersion, PdmUsageError, ProjectError
-from pdm.models.backends import BuildBackend, get_backend_by_spec
+from pdm.models.backends import DEFAULT_BACKEND, BuildBackend, get_backend_by_spec
 from pdm.models.caches import PackageCache
 from pdm.models.python import PythonInfo
 from pdm.models.repositories import BaseRepository, LockedRepository
 from pdm.models.requirements import Requirement, parse_requirement, strip_extras
 from pdm.models.specifiers import PySpecSet
-from pdm.project.config import Config
+from pdm.project.config import Config, ensure_boolean
 from pdm.project.lockfile import Lockfile
 from pdm.project.project_file import PyProject
 from pdm.utils import (
@@ -33,6 +34,7 @@ from pdm.utils import (
     find_python_in_path,
     is_conda_base,
     is_conda_base_python,
+    is_path_relative_to,
     path_to_url,
 )
 
@@ -202,6 +204,11 @@ class Project:
             if not self.is_global:
                 self.core.ui.info(message)
 
+        def is_active_venv(python: PythonInfo) -> bool:
+            if not (venv := os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))):
+                return False
+            return is_path_relative_to(python.executable, venv)
+
         config = self.config
         saved_path = self._saved_python
         if saved_path and not os.getenv("PDM_IGNORE_SAVED_PYTHON"):
@@ -215,11 +222,12 @@ class Project:
                 )
             self._saved_python = None  # Clear the saved path if it doesn't match
 
-        if config.get("python.use_venv") and not self.is_global and not os.getenv("PDM_IGNORE_ACTIVE_VENV"):
+        if config.get("python.use_venv") and not self.is_global:
             # Resolve virtual environments from env-vars
+            ignore_active_venv = ensure_boolean(os.getenv("PDM_IGNORE_ACTIVE_VENV"))
             venv_in_env = os.getenv("VIRTUAL_ENV", os.getenv("CONDA_PREFIX"))
             # We don't auto reuse conda's base env since it may cause breakage when removing packages.
-            if venv_in_env and not is_conda_base():
+            if not ignore_active_venv and venv_in_env and not is_conda_base():
                 python = PythonInfo.from_path(get_venv_python(Path(venv_in_env)))
                 if match_version(python):
                     note(
@@ -230,7 +238,7 @@ class Project:
             # otherwise, get a venv associated with the project
             for _, venv in iter_venvs(self):
                 python = PythonInfo.from_path(venv.interpreter)
-                if match_version(python):
+                if match_version(python) and not (ignore_active_venv and is_active_venv(python)):
                     note(f"Virtualenv [success]{venv.root}[/] is reused.")
                     self.python = python
                     return python
@@ -390,7 +398,7 @@ class Project:
         for source in result.values():
             if not source.url:
                 continue
-            source.url = expand_env_vars_in_auth(source.url)
+            source.url = DEFAULT_BACKEND(self.root).expand_line(expand_env_vars_in_auth(source.url))
             sources.append(source)
         return sources
 
@@ -562,6 +570,7 @@ class Project:
         to_group: str = "default",
         dev: bool = False,
         show_message: bool = True,
+        write: bool = True,
     ) -> None:
         deps, setter = self.use_pyproject_dependencies(to_group, dev)
         for _, dep in requirements.items():
@@ -575,7 +584,8 @@ class Project:
             else:
                 deps[matched_index] = req
         setter(cast(Array, deps).multiline(True))
-        self.pyproject.write(show_message)
+        if write:
+            self.pyproject.write(show_message)
 
     def init_global_project(self) -> None:
         if not self.is_global or not self.pyproject.empty():
@@ -725,6 +735,12 @@ class Project:
         from pdm.cli.commands.venv.utils import VenvProvider
 
         providers: list[str] = self.config["python.providers"]
+        venv_pos = -1
+        if not providers:
+            venv_pos = 0
+        elif "venv" in providers:
+            venv_pos = providers.index("venv")
+            providers.remove("venv")
         old_rye_root = os.getenv("RYE_PY_ROOT")
         os.environ["RYE_PY_ROOT"] = os.path.expanduser(self.config["python.install_root"])
         try:
@@ -734,8 +750,7 @@ class Project:
                 os.environ["RYE_PY_ROOT"] = old_rye_root
             else:
                 del os.environ["RYE_PY_ROOT"]
-        if search_venv and (not providers or "venv" in providers):
-            venv_pos = providers.index("venv") if providers else 0
+        if search_venv and venv_pos >= 0:
             finder.add_provider(VenvProvider(self), venv_pos)
         return finder
 
@@ -750,3 +765,22 @@ class Project:
             return settings["distribution"]
         else:
             return True
+
+    def get_setting(self, key: str) -> Any:
+        """
+        Get a setting from its dotted key (without the `tool.pdm` prefix).
+
+        Returns `None` if the key does not exists.
+        """
+        try:
+            return reduce(operator.getitem, key.split("."), self.pyproject.settings)
+        except tomlkit.exceptions.NonExistentKey:
+            return None
+
+    def env_or_setting(self, var: str, key: str) -> Any:
+        """
+        Get a value from environment variable and fallback on a given setting.
+
+        Returns `None` if both the environment variable and the key does not exists.
+        """
+        return os.getenv(var.upper()) or self.get_setting(key)
